#include "Person.h"
#include <algorithm>
#include <ios>
std::ostream& operator << (std::ostream& out, const Person& data) {
	for (auto i = data.cont.begin(); i != data.cont.end(); ++i) {
		out << i->full_name << " " <<
			i->enrollment_date << " " <<
			i->personnel_number << " " <<
			i->department_number << " " <<
			i->salary << " " <<
			i->bonus_percentage << " " <<
			i->income_tax << " " <<
			i->number_of_days_worked << " " << std::endl;
	}
	return out;
}

std::istream& operator >> (std::istream& in, Person& data) {
	data = Person();
	while (true) {
		Person::Data d;
		in >> d.full_name;
		in >> d.enrollment_date;
		if (d.full_name == "\\0" && d.enrollment_date == "\\0" || d == Person::Data()) {
			data.cont.insert(Person::Data());
			break;
		}
		in >> d.personnel_number;
		in >> d.department_number;
		in >> d.salary;
		in >> d.bonus_percentage;
		in >> d.income_tax;
		in >> d.number_of_days_worked;
		data.cont.insert(d);
	}
	return in;
}

bool Person::operator == (const Person& data) {
	if (cont.size() != data.cont.size())
		return false;
	auto i = cont.begin();
	auto j = data.cont.begin();
	while (i != cont.end() && j != data.cont.end()) {
		if (const_cast<Person::Data&>(*i) != const_cast<Person::Data&>(*j))
			return false;
		++i;
		++j;
	}
	return true;
}

void Person::add(std::ofstream& file) {
	for (auto i = cont.begin(); i != cont.end(); ++i) {
		addToFile(file, i);
	}
}
void Person:: addToFile(std::ofstream& file,std::multiset<Person::Data>::iterator i) {
	file.write(i->full_name.c_str(), strlen(i->full_name.c_str()));
	file.write(i->enrollment_date.c_str(), strlen(i->enrollment_date.c_str()));
	file.write(reinterpret_cast<const char*>(&i->personnel_number), sizeof(i->personnel_number));
	file.write(reinterpret_cast<const char*>(&i->department_number), sizeof(i->department_number));
	file.write(reinterpret_cast<const char*>(&i->salary), sizeof(i->salary));
	file.write(reinterpret_cast<const char*>(&i->bonus_percentage), sizeof(i->bonus_percentage));
	file.write(reinterpret_cast<const char*>(&i->income_tax), sizeof(i->income_tax));
	file.write(reinterpret_cast<const char*>(&i->number_of_days_worked), sizeof(i->number_of_days_worked));
}
void Person::read(std::ifstream& file) {
	while (!file.eof()) {
		Person::Data d;
		file.read(const_cast<char*>(d.full_name.c_str()), sizeof(d.full_name));
		file.read(const_cast<char*>(d.enrollment_date.c_str()), sizeof(d.enrollment_date));
		file.read(reinterpret_cast<char*>(&d.personnel_number), sizeof(d.personnel_number));
		file.read(reinterpret_cast<char*>(&d.department_number), sizeof(d.department_number));
		file.read(reinterpret_cast<char*>(&d.salary), sizeof(d.salary));
		file.read(reinterpret_cast<char*>(&d.bonus_percentage), sizeof(d.bonus_percentage));
		file.read(reinterpret_cast<char*>(&d.income_tax), sizeof(d.income_tax));
		file.read(reinterpret_cast<char*>(&d.number_of_days_worked), sizeof(d.number_of_days_worked));
		this->cont.insert(d);
	}
}

void Person::create_file(std::ofstream& file) {
	Person data;
	std::cin >> data;
	data.add(file);
}

void Person::delete_data(std::ofstream& file_write, std::ifstream& file_read) {
	Person data;
	data.read(file_read);
	bool b = true;
	for (auto i = data.cont.begin(); i != data.cont.end(); ++i) {
		if (b && !const_cast<Data&>(*i).in(this->cont))
			data.add(file_write);
		else
			b = false;
		if (!b)
			data.add(file_write);
	}
}

void Person::replacement(std::ofstream& file_write, std::ifstream& file_read, Person& data) {
	this->delete_data(file_write, file_read);
	data.add(file_write);
}

void Person::convert_to_txt(const std::string& filepath) {
	Person data;
	std::ifstream file_read(filepath + ".bin");
	data.read(file_read);
	if (rename((filepath + ".bin").c_str(), (filepath + ".txt").c_str()) != 0)
		throw std::exception("Wrong of file!");
	std::ofstream file_write(filepath + ".txt");
	file_write << data;
}
void Person::find_full_name(const std::string& p, std::ofstream& file)
{
	for (auto i = cont.begin(); i != cont.end(); ++i) {
		if (i->full_name == p) {
			addToFile(file, i);
		}
	}

}

void Person::find_department_number(int p, std::ofstream& file)
{
	for (auto i = cont.begin(); i != cont.end(); ++i) {
		if (i->department_number == p) {
			addToFile(file, i);
		}
	}

}

void Person::find_enrollment_date(const std::string& p, std::ofstream& file)
{
	for (auto i = cont.begin(); i != cont.end(); ++i) {
		if (i->enrollment_date == p) {
			addToFile(file, i);
		}
	}

}