#pragma once
#include <fstream>
#include <set>
#include <iostream>

class Person {
private:
	struct Data {
		std::string full_name;
		std::string enrollment_date;
		int personnel_number;
		int department_number;
		int salary;
		int bonus_percentage;
		int income_tax;
		int number_of_days_worked;
		

		Data() :
			full_name(""),
			enrollment_date(""),
			personnel_number(0),
			department_number(0),
			salary(0),
			bonus_percentage(0),
			income_tax(0),
			number_of_days_worked(0)
		{}

		Data(std::string _full_name,
			std::string _enrollment_date,
			int _personnel_number,
			int _department_number,
			int _salary,
			int _bonus_percentage,
			int _income_tax,
			int _number_of_days_worked) :
			full_name(_full_name),
			enrollment_date(_enrollment_date),
			personnel_number(_personnel_number),
			department_number(_department_number),
			salary(_salary),
			bonus_percentage(_bonus_percentage),
			income_tax(_income_tax),
			number_of_days_worked(_number_of_days_worked)
		{}


		bool operator == (const Data& data) {
			return (this->full_name == data.full_name &&
				this->enrollment_date == data.enrollment_date &&
				this->personnel_number == data.personnel_number &&
				this->department_number == data.department_number &&
				this->salary == data.salary &&
				this->bonus_percentage == data.bonus_percentage &&
				this->income_tax == data.income_tax &&
				this->number_of_days_worked == data.number_of_days_worked);
		}
		bool operator != (const Data& data) {
			return !(*this == data);
		}

		friend bool operator < (const Data& data, const Data& data2) {
			return data.number_of_days_worked < data2.number_of_days_worked;
		}

		bool in(const std::multiset<Data> cont) {
			for (auto i = cont.begin(); i != cont.end(); ++i) {
				if (*this == *i)
					return true;
			}
			return false;
		}

	};
	std::multiset<Data> cont;

public:

	Person() : cont(std::multiset<Data>()) {}

	Person(std::multiset<Data> cont) : cont(cont) {}

	bool operator == (const Person&);

	bool operator != (const Person& data) {
		return !(*this == data);
	}

	friend std::ostream& operator << (std::ostream&, const Person&);
	friend std::istream& operator >> (std::istream&, Person&);

	void add(std::ofstream&);
	void addToFile(std::ofstream& file, std::multiset<Person::Data>::iterator);
	void read(std::ifstream&);
	static void create_file(std::ofstream&);

	void delete_data(std::ofstream&, std::ifstream&);
	void replacement(std::ofstream&, std::ifstream&, Person&);
	static void convert_to_txt(const std::string& filepath);
	void find_full_name(const std::string& p, std::ofstream& file);
	void find_department_number(int p, std::ofstream& file);
	void find_enrollment_date(const std::string& p, std::ofstream& file);
};
